/*
 * Copyright (C) 2001-2004 by Andrei Alexandrescu
 * This code accompanies the book:
 * Alexandrescu, Andrei. "Modern C++ Design: Generic Programming and Design
 *    Patterns Applied". Copyright (c) 2001. Addison-Wesley.
 *
 * Permission to use, copy, modify, distribute and sell this software for any
 *     purpose is hereby granted without fee, provided that the above copyright
 *     notice appear in all copies and that both that copyright notice and this
 *     permission notice appear in supporting documentation.
 * The author or Addison-Wesley Longman make no representations about the
 *     suitability of this software for any purpose. It is provided "as is"
 *     without express or implied warranty.
 *
 */
 
#ifndef _L1394_SINGLETON_HPP
#define _L1394_SINGLETON_HPP

#include <stdlib.h>

/**
  */
namespace L1394 {

/**
  */


template <class T>
class SingleThreaded {
public:
  typedef  T VolatileType;
};

template <class T>
class MultiThreaded {
public:
  typedef T VolatileType;
};

template <class T>
class CreateUsingNew {
public:
  static T* create()           {return new T();}
  static void destroy(T* obj)  {delete obj;}
};

template <class T>
class DefaultLifetime {
public:
  typedef void (*deleteMethod)();
  static void scheduleDestruction( deleteMethod d) { atexit(d); }
  static void onDeadReference()                   { /*throw std::logic_error("Dead Reference Detected");*/ }
};

template <
  class T,
  template <class> class CreationPolicy = CreateUsingNew,
  template <class> class LifetimePolicy = DefaultLifetime,
  template <class> class ThreadingModel = MultiThreaded
>
class Singleton {
public:
  static T* getInstance();

protected:
  Singleton(){};

private:
  static void destroySingleton();
  typedef typename ThreadingModel<T>::VolatileType InstanceType;
  static T* _instance;
  static  bool destroyed;
};



/** Implementation */

template <
  class T,
  template <class> class CreationPolicy, //creation policy
  template <class> class LifetimePolicy, //Lifetime policy
  template <class> class ThreadingModel  //Threading model
>
T* Singleton<T, CreationPolicy, LifetimePolicy, ThreadingModel>::_instance = 0;

template <
  class T,
  template <class> class CreationPolicy, //creation policy
  template <class> class LifetimePolicy, //Lifetime policy
  template <class> class ThreadingModel  //Threading model
>
bool Singleton<T, CreationPolicy, LifetimePolicy, ThreadingModel>::destroyed = false;


template <
  class T,
  template <class> class CreationPolicy, //creation policy
  template <class> class LifetimePolicy, //Lifetime policy
  template <class> class ThreadingModel  //Threading model
>
T* Singleton<T, CreationPolicy, LifetimePolicy, ThreadingModel>::getInstance() {
  if (!_instance) {
//    typename ThreadingModel::Lock guard;
    if (!_instance) {
      if (destroyed) {
        LifetimePolicy<T>::onDeadReference();
        destroyed = false;
      }
      _instance = CreationPolicy<T>::create();
      LifetimePolicy<T>::scheduleDestruction(&destroySingleton);
    }
  }
  return _instance;
}

template <
  class T,
  template <class> class CreationPolicy, //creation policy
  template <class> class LifetimePolicy, //Lifetime policy
  template <class> class ThreadingModel  //Threading model
>
void Singleton<T, CreationPolicy, LifetimePolicy, ThreadingModel>::destroySingleton() {
  CreationPolicy<T>::destroy(_instance);
  _instance = 0;
  destroyed = true;
}
}

#endif
