
#ifndef _XSHMIMG_H
#define _XSHMIMG_H

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#include "l1394_window.h"


namespace L1394_MM{
/*! \class XShmImg
  *  \ingroup L1394_Display
  * \brief This class implements a X-Window with shared memory.
  *
  */

class XShmImg : public Window {
public:

/*! \fn XShmImg(const int width, const int height, const char *title, l1394_converter converter)
  * \brief constructor
  * \param width : width value for the window
  * \param height : height value for the window
  * \param title : char* title of the window
  *  \param converter : function pointer to the decoder (will be removed in later versions)
  */
  XShmImg(const int width,const int height,const char *title, l1394_converter=0);

/*! \fn XShmImg(const char *title, l1394_converter converter)
  * \brief constructor
  * \param title : char* title of the window
  *  \param converter : function pointer to the decoder (will be removed in later versions)
  */
  XShmImg(const char *title, l1394_converter=0);

/*! \fn ~XShmImg()
  * \brief destructor
  */
  virtual ~XShmImg();

/*! \fn getBuffer() const
  * \brief This function returns a pointer to the shared memory of the window
  */
  unsigned char* getBuffer() const;

/*! \fn update()
  * \brief This function updates the X-window.
  */
  void update();

/*! \fn setTitle(const char *title)
  * \brief This function sets the title of the X-window.
  * \param title : const char* to the name of the window.
  */
  void setTitle(const char *title);


/*! \fn getColorDepth() const
  * \brief This function returns the color depth of a window as integer value.
  *
  * This value depends on your X-Server settings.
  * \return int : the color depth (in Bit) as integer value (normally 16 o 24)
  */
  int getColorDepth() const {return depth;}
  virtual  void  start();

/*! \fn nodeDestroy(const L1394::Node* node)
  *  \brief If this function is called, the destructor of this class is called.
  *
  * So you don't need to delete a connected window.
  *  \param node: pointer to the connected parent-node
  */
  virtual  void nodeDestroy(const L1394::Node*);

/*! \fn nodeDisabled(const L1394::Node* node)
  *  \brief This function handles the NODE_DISABLED event from the connected device.
  */
  virtual void nodeDisabled(const L1394::Node*);
private:
  void allocate();
  inline void setPixel(int x, int y, char r, char g, char b)
    {
      unsigned long pixel = r;
      pixel <<= 8;
      pixel |= g;
      pixel <<= 8;
      pixel |= b;
      XPutPixel(image, x, y, pixel);
    }

  int width, height, depth;

  Display *display;
  ::Window win;
  GC gc;
  XImage *image;
  XShmSegmentInfo Shminfo;
  bool is_init;
  const char* title;
};
}
#endif
