/***************************************************************************
                          l1394businfo.h  -  description
                             -------------------
    begin                : Fri Oct 27 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394BUSINFO_H
#define L1394BUSINFO_H

#include "l1394_qarray.h"
#include <iostream>
namespace L1394{
namespace internal{
/*! \class BusInfo
  * \ingroup Internal
  * \brief This class stores the BusInfo block of the control and status register(CSR)
  *  defined in IEEE 1212.
  *
  *  Every Node has a BusInfo object that stores its capabilities.
  * For more information see IEEE 1212 specification.
  *
  * @author Michael Repplinger
  */

class BusInfo{

public:

/*! \fn BusInfo(QArray *)
  * \brief Constructor
  * \param QArray* pointer to the bus information block
  */
  BusInfo(QArray *);


/*!  \fn ~BusInfo()
  * \brief Destructor
  */
  ~BusInfo();

/*! \fn getNodeInfo() const
  * \brief return the complete node information Quadlet
  */
  Quadlet getNodeInfo() const {return node_info;}

/*! \fn getIrmc() const
  *  \brief return 1 if node is Iso Resource Manager Capable, else 0
  */
  int getIrmc() const       {return node_info.getBit(31);}

/*! \fn getCmc() const
  * \brief return 1 if node is Cycle Master Capable, else 0
  */
  int getCmc()  const        {return node_info.getBit(30);}

/*! \fn getIsc() const
  * \brief return 1 if node Iso Capable, else 0
  */
  int getIsc()  const        {return node_info.getBit(29);}

/*! \fn getBmc() const
  * \brief return 1 if node is Bus Master Capable, else 0
  */
  int getBmc()  const        {return node_info.getBit(28);}

//check if correct bit will be returned

/*! \fn getCykClk() const
  * \brief return the Cycle Clock
  */
  int getCykClk()  const      {return node_info.getBitRange(16,23);}

/*! \fn getMaxRec() const
  * \brief return maximum record
  */
  int getMaxRec()  const      {return node_info.getBitRange(12,15);}

/*! \fn getChipIdLow() const
  * \brief return the ChipIdLow (last 32 Bit of GUID)
  */
  int getChipIdLow() const   {return chip_id_low.toInt();}

/*! \fn getChipIdHi() const
  * \brief return the chipIdHi (the last 8Bit from the first 32 Bit part of GUID)
 */
  int getChipIdHi()  const    {return node_info.getBitRange(24,31);}

/*! \fn getNodeVendorId() const
   * \brief return the node vendor id (the first 24 Bit of GUID)
 */
  int getNodeVendorId() const  {return node_vendor_id.getBitRange(0,23);}

/*! \fn getBusName() const
  * \brief return the bus name (should be "1394")
  */
  const char* getBusName() const {return bus_name;}


  friend ostream& operator<<(ostream& o, const BusInfo&);

private:
  BusInfo(const BusInfo&);
  int romInfoBlockLength;
  char* bus_name;

  Quadlet node_info, node_vendor_id, chip_id_low;
};
}
}
#endif
