/***************************************************************************
                          l1394bustopology.h  -  description
                             -------------------
    begin                : Thu Jul 20 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394BUSTOPOLOGY_H
#define L1394BUSTOPOLOGY_H

#include "l1394_toponode.h"
#include "l1394_node.h"
#include "l1394_message.h"

namespace L1394{
namespace internal{
/*! \class BusTopology
  * \ingroup Internal
  * \brief This class stores the topology map.
  *
  *  The topology map represent the actual configuration of the FireWire bus.
  * All Card objects use this class to get information about the topology of the
  * bus after a bus reset.<BR>
  *  This class translate the topology map in a topology-tree. To access the topology
  * tree use the method getRootNode().
  *
  * @author Michael Repplinger
  */


class BusTopology {

public:

/*! \fn BusTopology(Node *parent)
  * \brief Constructor
  * \param Node* pointer to parent node.
  */
  BusTopology(Node *parent);


/*! \fn ~BusTopology()
  * \brief Destructor
  */
  ~BusTopology();

/*! \fn readTopoMap()
  * \brief This method reads the topology map and returns a pointer to it.
  *
  * If you call this method the old topology map is deleted and a new one is created.
  * \return QArray* : pointer to the topomap
  */
  QArray* readTopoMap();

/*! \fn getRootNode()
  * \brief Creates the topology tree and returns the root node;
  */
  internal::TopoNode* getRootNode();

  void print();


private:
  BusTopology(const BusTopology&);
  internal::TopoNode* makeTree(internal::TopoNode *);
  QArray *rom_info;  int rom_length; int node;
  internal::TopoNode *root_node;
  int start;
  Node *parent;
  Message*  message;
};
}
}
#endif
