/***************************************************************************
                          converter.h  -  description
                             -------------------
    begin                : Thu Nov 30 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONVERTER_H
#define CONVERTER_H
#define CLIPPED(x) ((x)>255?255:((x)<0?0:(x)))
typedef int (*l1394_converter)(unsigned char*, int,unsigned char*);
//extern "C" { void unpack422rgb32_mmx(unsigned char *src, int n, unsigned char *dst);}

#include <string>
#include <map>

using std::string;
using std::map;

namespace L1394_MM
{

/*! \class Converter
  *  \ingroup L1394_Display
  * \brief This class provides functions to convert image formats.
  *
  * The idea of this class is to say which source and destination format yo need, and this
  * class returns a pointer to the correct converter routine. <BR>
  * To add your own Converter functions inherit from this class and implement your own Converter routines.
  * All Converter routines have the same parameter.
  * The first parameter is an unsigned char pointer to the source array. The second parameter is
  * an integer value with the size of the source array and the third parameter
  * is an unsigned char pointer to the destination array. The destination array must have correct size.<BR>
  *
  * This class is still under develop. The converter routines are still public. Use
  * these functions instead the getConverter() function.
  *@author Michael Repplinger
  */

class Converter {
public:
/*! \fn Converter()
  * \brief constructor
  */
  Converter();

/*! \fn ~Converter()
  * \brief destructor
*/
  virtual ~Converter();

/*! \fn getConverter(string source_format, string destination_format, bool mmx)
  * \brief With this function you can get a pointer to Converter routine you need.
  *
  * You need to cast the void pointer to a Converter_t. Example:
  * \code
  * Converter c;
  * l1394_converter converter = (l1394_converter) c.getConverter(YUV_422, RGB, 24, true);
  * converter(source_pointer, source_size, destination_pointer);
  * \endcode
  * \param source_format : source format of your image
  * \param destination_format : format you will decode the image
  * \param color_depth : color depth of your destination format
  * \param mmx : if true a mmx optimized function will be returned (if one exist), else a normal C-routine.
  * \return void* : pointer to the converter function, or NULL, if no converter exist
  */
  virtual l1394_converter getConverter(string source_format, string destination_format, bool mmx);

  static int UYVYtoBGRA32(unsigned char* source, int source_size,unsigned char* destination);
  //static int UYVYtoBGRA32_MMX(unsigned char* source, int source_size,unsigned char* destination);
  static int RGB24toBGRA32(unsigned char* source, int source_size,unsigned char* destination);
  private:
  void createMap();
  map<string,l1394_converter> converter_map;

};
}
#endif
