/***************************************************************************
                          l1394csrrom.h  -  description
                             -------------------
    begin                : Thu Sep 21 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394CSRROM_H
#define L1394CSRROM_H


#include <netinet/in.h>

#include "l1394_businfo.h"
#include "l1394_csrdirectory.h"
#include "l1394_node.h"
//document_status : final
//debug_status : tested

namespace L1394{
namespace internal{
/*! \class CsrRom
  * \ingroup Internal
  * \brief This class create an CsrRom Object, defined in the IEEE1212 standard.
  *
  * Every node on FireWire bus, expect nodes with a physical layer only, has a Control
  * and Status Register (CSR) rom. So very Node object has a CsrRom object, expect the PhyNode objects.
  *
  * The main object in CSR rom is the CSR-directory, which (can) contain a lot of information
  * about the FireWire node. About datas and data structure that can be stored in
  * CSR rom see the IEEE-1212 specification.
  *
  * So this class provide functions to access the CsrDirectory object and the bus information block
  * of a node.
  *
  * @author Michael Repplinger
  */


class CsrRom
{

public:


/*! \fn CsrRom(Node* parent)
  * \brief Constructor
  * \param parent : pointer to parent node
  */
  CsrRom(Node* parent);

 /*! \fn ~CsrRom()
   * \brief destructor
   */
  ~CsrRom();

/*! \fn getBusInfoBlock()
  * \brief This method returns the bus information block of a node.
  *
  * The bus information block store information about general features of
  * a node, like isMasterCapable and so on. The BusInfo object will be created
  * by the first call of this function. For more information see class BusInfo.
  * \return BusInfo* : pointer to the bus information block.
  */
  BusInfo* getBusInfoBlock()  { if (node_bus_info == NULL)
                                readInfoBlock (); return node_bus_info; }

/*! \fn getCsrDirectory()
  * \brief With this method you can access the csr directory of an node.
  *
  * The CsrDirectory object will be created by the first call of this function.
  * For more information see class CsrDirectory.
  * \return CsrDirectory* : pointer
  */
  CsrDirectory *getCsrDirectory() { if (root_directory == NULL) readDirectory(); return root_directory;}


private:

  //disabled copy constructor
  CsrRom(const CsrRom&);

  // Read the bus information block
  void readInfoBlock();

  // Read the directory information
  void readDirectory();

  // Read the bus topology from a master capable device
  void storeBlockInfoValues(QArray*);
  void storeDirectoryValues(int);
  void storeDirectory(int, int);
  void storeLeaf(int, int);

  BusInfo *node_bus_info;

  CsrDirectory *root_directory, *actual_directory;

  Node *parent;

};
}
}
#endif
