/***************************************************************************
                          dcciso.h  -  description
                             -------------------
    begin                : Fri May 4 2001
    copyright            : (C) 2001-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394_DCCISO_H
#define L1394_DCCISO_H

#include "l1394_iso.h"

#include <netinet/in.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <string>
#include "libraw1394/raw1394.h"
#include "libraw1394/csr.h"
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/mman.h>
#include "Thread.hpp"


using std::string;

namespace L1394{
namespace internal{
/**
  *@author Michael Repplinger
  */
class DccIso : public Iso{
public: 
  DccIso(const char *device);
  virtual ~DccIso();
  virtual void releaseFrame(Frame* );
  virtual Frame *getFrame();
  virtual void setFrameMode(bool b) {release_mode = b;}
  virtual bool getFrameMode() const {return release_mode;}
  virtual int  startIsoListen()     ;
  virtual int  stopIsoListen()      ;

  virtual int  setParameter(const int buffercount, const int channel, const int buffersize);
  virtual void flushQueue();
private: //private structs and enums

#define VIDEO1394_SYNC_FRAMES          0x00000001
#define VIDEO1394_INCLUDE_ISO_HEADERS  0x00000002
#define VIDEO1394_VARIABLE_PACKET_SIZE 0x00000004
#define VIDEO1394_MAX_SIZE 0x4000000
#define VIDEO1394_IOC_LISTEN_CHANNEL		\
	_IOWR('#', 0x10, struct video1394_mmap)
#define VIDEO1394_IOC_UNLISTEN_CHANNEL		\
	_IOW ('#', 0x11, int)
#define VIDEO1394_IOC_LISTEN_QUEUE_BUFFER	\
	_IOW ('#', 0x12, struct video1394_wait)
#define VIDEO1394_IOC_LISTEN_WAIT_BUFFER	\
	_IOWR('#', 0x13, struct video1394_wait)
#define VIDEO1394_IOC_TALK_CHANNEL		\
	_IOWR('#', 0x14, struct video1394_mmap)
#define VIDEO1394_IOC_UNTALK_CHANNEL		\
	_IOW ('#', 0x15, int)
#define VIDEO1394_IOC_TALK_QUEUE_BUFFER 	\
	_IOW ('#', 0x16, size_t)
#define VIDEO1394_IOC_TALK_WAIT_BUFFER		\
	_IOW ('#', 0x17, struct video1394_wait)
#define VIDEO1394_IOC_LISTEN_POLL_BUFFER	\
	_IOWR('#', 0x18, struct video1394_wait)
enum {
  VIDEO1394_BUFFER_FREE = 0,
  VIDEO1394_BUFFER_QUEUED,
  VIDEO1394_BUFFER_READY
};

enum {
  VIDEO1394_LISTEN_CHANNEL = 0,
  VIDEO1394_UNLISTEN_CHANNEL,
  VIDEO1394_LISTEN_QUEUE_BUFFER,
  VIDEO1394_LISTEN_WAIT_BUFFER,  // wait until buffer is ready
  VIDEO1394_TALK_CHANNEL,
  VIDEO1394_UNTALK_CHANNEL,
  VIDEO1394_TALK_QUEUE_BUFFER,
  VIDEO1394_TALK_WAIT_BUFFER,
  VIDEO1394_LISTEN_POLL_BUFFER   // return immediately with -EINTR if not ready
};


struct video1394_mmap {
  int channel;      /* -1 to find an open channel in LISTEN/TALK */
  unsigned int sync_tag;
  unsigned int nb_buffers;
  unsigned int buf_size;
  unsigned int packet_size; /* For VARIABLE_PACKET_SIZE:
             Maximum packet size */
  unsigned int fps;
  unsigned int syt_offset;
  unsigned int flags;
};

/* For TALK_QUEUE_BUFFER with VIDEO1394_VARIABLE_PACKET_SIZE use */
struct video1394_queue_variable {
  unsigned int channel;
  unsigned int buffer;
  unsigned int* packet_sizes; /* Buffer of size:
               buf_size / packet_size  */
};

struct video1394_wait {
  unsigned int channel;
  unsigned int buffer;
  struct timeval filltime;  /* time of buffer full */
};

private:
  ThreadMutex* mutex;

  int  fd;
  int  old_buffer;
  int  new_buffer;

  bool release_mode;
  bool is_init;
  video1394_mmap video_mmap;

  unsigned char* map;

  Frame** frame_array;
  Frame** unreleased_buffer;

  string device;

  static ThreadMutex** mutex_array;
private:
  void releaseBuffer(int i)  ;
  void listenFrame(Frame*)   ;
  void init(video1394_mmap* );
  void makeClean();

};
}
}
#endif
