/***************************************************************************
                          dviso.h  -  description
                             -------------------
    begin                : Fri May 4 2001
    copyright            : (C) 2001-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DVISO_H
#define DVISO_H

#include "l1394_iso.h"
#include "Thread.hpp"
#include "libraw1394/raw1394.h"
#include <deque>

using std::deque;

namespace L1394{
class Node;
namespace internal{

/**
  *@author Michael Repplinger
  */
class DvIso : public Iso{

public:
  DvIso(const Node* parent, raw1394handle_t);
  ~DvIso();
  virtual Frame* getFrame();
  virtual void releaseFrame(Frame*);
  virtual bool getFrameMode()       const {return frame_mode;}
  virtual void setFrameMode(const bool b) {frame_mode = b;}

  virtual int setParameter(const int buffercount,const int channel,  const int buffersize);

/*!\fn startIsoListen()
 * \brief This function starts iso listening on a specific channel
 */
  int startIsoListen();

/*!\fn stopIsoListen()
 * \brief This function stops iso listening.
 */
  int stopIsoListen();

private:
  void isoRun();

/*!\fn L1394IsoHandle(raw1394handle_t handle, int channel, size_t length,  quadlet_t *data)
 * \brief This function is the default iso handler for this library
 */
  static int L1394IsoHandle(raw1394handle_t handle, int channel, size_t length,  quadlet_t *data);

  static void* run_static(void *me);
  static DvIso** dv_iso_array;

  pthread_t thread;
  ThreadMutex mutex;

  int   channel;
  int   buffer_count;
  bool  iso_listen;
  bool  frame_mode;
  const Node* parent;

  deque < Frame* > frame_queue;
  deque < Frame* > ready_frames;
  Frame* current_frame;

  raw1394handle_t iso_handler;
};
}
}
#endif
