#ifndef __ISO_H
#define __ISO_H

#define VIDEO1394_SYNC_FRAMES         0x00000001
#define VIDEO1394_INCLUDE_ISO_HEADERS 0x00000002
#define VIDEO1394_VARIABLE_PACKET_SIZE 0x00000004


#include "l1394_frame.h"
#include "l1394_resource.h"
namespace L1394{


/*! \class Iso
  * \brief This abstract class defines the interface for iso-transmissions.
  *
  * The method getBuffer returns the actual buffer with one frame.
  *  At this time the method getBuffer() blocks until a buffer is filled
   * with video datas.
   *
   * The actual version of the video module is not multi-thread safe!!!
  */


class Iso
{
public:

/*! \fn ~Iso()
  * \brief destructor
  */
  virtual ~Iso(){};

/*! \fn getFrame()
  * \brief This method returns a pointer to the next incoming frame.
  *
  * If no complete frame is available this method (blocked) waits
  * for the next frame. The Frame will be released by calling this
  * method again, if the auto-release mode is enabled.
  *
  * \return Frame* pointer to the buffer with the video frame.
  *
  */
  virtual Frame *getFrame()  = 0;

/** This method releases a Frame.
  */
  virtual void releaseFrame(Frame* frame) = 0;

/* This method enables or disables the auto-release mode. By default, the auto-release mode
 * is enabled
 */
  virtual void setFrameMode(const bool b) = 0;

/** Returns the current frame mode
  */
  virtual bool getFrameMode() const       = 0;

/** sets the parameter
  */
  virtual int  setParameter(const int buffercount, const int channel, const int buffersize) = 0;

/** This method starts iso-receive
  */
  virtual int  startIsoListen() = 0;

/** This method stops iso-receive
  */
  virtual int  stopIsoListen () = 0;
  virtual void flushQueue() {}
};

}
#endif



