/***************************************************************************
                          l1394_message.h  -  description
                             -------------------
    begin                : Wed Mar 7 2001
    copyright            : (C) 2001-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394_MESSAGE_H
#define L1394_MESSAGE_H

#include <fstream>
#include "Thread.hpp"
#include "Singleton.hpp"


namespace L1394 {
class Node;

namespace internal{
/** This class provides 4 streams for the different output messages (error-, warning-, debug- and simple messages)
  * All streams can be redirect to any other valid stream, like a file, or can be disabled.
  *
  *@author Michael Repplinger
  */

class Message {
friend class CreateUsingNew<Message>;
public: 
  void setErrorStream(std::ostream* o);
  void setDebugStream(std::ostream* o);
  void setWarningStream(std::ostream* o);
  void setMessageStream(std::ostream* o);

  std::ostream& errorStream() const;
  std::ostream& debugStream() const;
  std::ostream& warningStream() const;
  std::ostream& messageStream() const;

  std::ostream& errorStream(const Node*) const;
  std::ostream& debugStream(const Node*) const;
  std::ostream& warningStream(const Node*) const;
  std::ostream& messageStream(const Node*) const;

private:
  Message();
  ~Message();
  Message(const Message&);

  std::ostream*  error_stream;
  std::ostream*  warning_stream;
  std::ostream*  debug_stream;
  std::ostream*  message_stream;
  std::ofstream* default_stream;
};
typedef Singleton<L1394::internal::Message, CreateUsingNew> SMessage;
}
}
#endif
