/***************************************************************************
                          l1394_phynode.h  -  description
                             -------------------
    begin                : Fri Jul 21 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394PHYNODE_H
#define L1394PHYNODE_H

#include "l1394_node.h"

namespace L1394{
/**\class PhyNode
  *\ingroup L1394_Node
  *\brief This class represents a node with physical layer only(like hubs)
  *
  * FireWire nodes with a physical layer only are normally hubs.
  * The read, write and lock functions are disabled for these nodes because the
  *  asynchrone transactions need a transaction layer.
  *  The GUID is set to 0 for these nodes.
  *
  * A PhyNode is deleted after every bus rest and create new if it's still
  * connected to the FireWire bus.(because there is no way to identify such a node)
  *
  *@author Michael Repplinger
  */

class PhyNode : public Node  {
public:

/*! \name PhyNode constructor
  * These functions creates the PhyNode objects.
  */
//@{
/*! \fn PhyNode(const u_int32_t node_id, const Card* parent)
  * \brief constructor
  * \param node_id : the current node_id by creating this object
  * \param parent  : pointer to the parent node
  */
  PhyNode(const u_int32_t , const Card* parent );

/*! \fn ~PhyNode()
  * \brief destructor
  */
  ~PhyNode();
//@}

/*! \fn isPhyOnly()
  * \brief This method tests if a node has only a physical layer.
  * \return bool: return true
 */
  virtual bool isPhyOnly(){return true;}

  //!disabled read functions
  virtual int read(const u_int64_t , Quadlet*) {return L1394_FAILED;}
  //!disabled write functions
  virtual int write(const u_int64_t ,const Quadlet&  ) {return L1394_FAILED;}
//  virtual int lock
  virtual Iso* getIsoObject() const {return 0;}
private:


};
}
#endif
