/***************************************************************************
                          l1394_timecode.h  -  description
                             -------------------
    begin                : Wed Jan 24 2001
    copyright            : (C) 2001-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394_TIMECODE_H
#define L1394_TIMECODE_H



namespace L1394{
typedef unsigned int u_int;

/*!  \class Timecode
  *  \brief This class represents a timecode.
  *
  * Main use of this class to get or set timecode from Vcrs.
  *
  *@author Michael Repplinger
  */
class Timecode {

public:

/*! \name Constructor
  * The following Constructor are available
  */
//@{
/*! \fn Timecode()
  *  \brief Constructor
  */
  Timecode();

/*! \fn Timecode(const Timecode&)
  *  \brief Copy Constructor
  *  \param t : reference to the Timecode
  */
  Timecode(const Timecode& ) ;

/*! \fn ~Timecode()
  *  \brief Destructor
  */
  ~Timecode();
//@}

/*! \name Manipulate the timecode
  * These functions set/get the values of a Timecode.
  */
//@{
/*! \fn setFrame(u_int frame)
  * \brief This method sets the frame
  */
  void setFrame(u_int frame)        {_frame = frame;}

/*! \fn setSecond(u_int second)
  * \brief This method sets the second
  */
  void setSecond(u_int second)      { _second = second;}

/*! \fn setMinute(u_int minute)
  * \brief This method functions sets the minute
  */
  void setMinute(u_int minute)      { _minute = minute;}

/*! \fn setHour(u_int hour)
  * \brief  This method sets the hour
  */
  void setHour(u_int hour)          {_hour = hour;}

/*! \fn setDay(u_int day)
  * \brief This method sets the day
  */
  void setDay(u_int day)            {_day = day;}

/*! \fn setWeek(u_int week)
  * \brief This method sets the week
  */
  void setWeek(u_int week)          {_week = week;}

/*! \fn setMonth(u_int month)
  * \brief This method sets the month
  */
  void setMonth(u_int month)        {_month = month;}

/*! \fn setYear(u_int year)
  * \brief This method sets the year
  */
  void setYear(u_int year)          {_year = year;}

/*! \fn setTimezone(u_int timezone)
  * \brief This method sets the timezone
  */
  void setTimezone(u_int timezone)  {_time_zone = timezone;}

/*! \fn frame() const
  *  \brief This method returns the frame
  */
  u_int frame()      const    {return _frame;}

/*! \fn second()    const
  * \brief This method returns the second.
  */
  u_int second()    const   {return _second;}

/*! \fn minute() const
  * \brief This method returns the minute
  */
  u_int minute()    const   {return _minute;}

/*! \fn hour() const
  * \brief This method returns the hour.
  */
  u_int hour()      const   {return _hour;}

/*! \fn day() const
  * \brief This method returns the day
  */
  u_int day()        const   {return _day;}

/*! \fn week() const
  * \brief This method returns week
  */
  u_int week()      const   {return _week;}

/*! \fn month() const
  * \brief This method returns the month.
  */
  u_int month()      const   {return _month;}

/*! \fn year() const
  * \brief This method returns the year.
  */
  u_int year()      const   {return _year;}

/*! \fn timezone() const
  * \brief This method returns the timezone
  */
  u_int timezone()  const   {return _time_zone;}
//@}

/*! \name Computing operators
  * These functions implements some computing operators
  */
//@{

/*! \fn operator=(const Timecode& t)
  * \brief  The operator= assigns the Timecode t to this Timecode and returns  a reference to it
  */
  Timecode& operator=(const Timecode& t);

//@}
private:
  u_int _frame;  //value between 0..30
  u_int _second; //value between 0..60
  u_int _minute; //value between 0..60
  u_int _hour;   //value between 0..24
  u_int _day;    //value between 0..7
  u_int _week;   //value between 0..3
  u_int _month;  //value between 0..12
  u_int _year;   //value between 0..65536 (shouldn't cause problems so far :-) )
  u_int _time_zone;

};



}//end namespace

#endif
