/***************************************************************************
                          node.h  -  description
                             -------------------
    begin                : Mon Jun 26 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NODE_H
#define NODE_H

#include <iostream>

#include "l1394_qarray.h"

namespace L1394{
namespace internal{

/*! \class TopoNode
  * \ingroup Internal
  * \brief A TopoNode represent an node in the topologie structure.
  *
  * A TopoNode store also information from the physical layer.
  * This value describe the information like speed, ports, powerclass
  * and so on.
  *
  * @author Michael Repplinger
  */


class TopoNode {

public:

/*! \fn TopoNode(int , TopoNode*)
  * \brief Constructor
  * \param int Integer Value that specify the port count
  * \param TopoNode* pointer to parent node
  */
  TopoNode(int , TopoNode*);

/*! \fn ~TopoNode()
  * \brief Destructor
  */
  ~TopoNode();

/*! \fn setChild(int i, TopoNode *node)
  * \brief add a child to a port
  *  \param int Integer value that specify the port
  * \param TopoNode* pointer to the childnode.
  */
  void setChild(int i, TopoNode *node){ if (i < max_port) child[i] = node; }

/*! \fn setPortStatus(int, unsigned short int)
*/
  void setPortStatus(int, unsigned short int);

/*! \fn getChild(int i)
  * \brief return a pointer to the i'th node
  * \param int Integer value that specify the port
  */
  TopoNode* getChild(int i) { if (i < max_port) return child[i]; else return NULL;}

/*! \fn unsigned short int getPortStatus(int i)
  * \brief
  */
  unsigned short int getPortStatus(int i) {if (i < max_port) return port_status[i]; else return 10;}

  int getMaxPort()     {return max_port;}
  int getPhyId()       {return node_info.getBitRange(24,29);}
  int getPhySpeed()   {return node_info.getBitRange(14,15);}
  int getPhyDelay()   {return node_info.getBitRange(12,13);}
  int getPwrClass()   {return node_info.getBitRange(8,10);}
  int getGapCount()   {return node_info.getBitRange(16,21);}
  int getLinkActive()  {return node_info.getBit(22);}

  void setNodeInfo(Quadlet *t);
  void printNode();
  const char* getPhySpeedSpec(int i) {return phy_speed_spec[i];}


  friend ostream& operator<<(ostream &, TopoNode&);
//  friend Html&    operator<<(Html&, TopoNode&);


  static  const char* phy_speed_spec[];
  static  const char* phy_delay_spec[];
  static  const char* pwr_class_spec[];
  static  const char* port_value[];

private:
  //disable copy constructor
  TopoNode(const TopoNode&);

  int max_port;

  Quadlet node_info;

  unsigned short int *port_status;
  TopoNode **child;
  TopoNode *parent;

};
}
}
#endif
